@@ -14,13 +14,15 @@ import numpy as np
 
 import tabular as tb
 import tabular.utils as utils
+import tempfile
 from tabular.colors import Point2HexColor
 
 __all__ = ['tabular2html']
 
-def tabular2html(fname, X=None, fin=None, title=None, printheader=False, 
+def tabular2html(fname=None, X=None, fin=None, title=None, printheader=False, 
                  split=True, usecss=None, writecss=None, SERVERNAME=None, 
-                 SERVER_FROM_CURDIR='../', ROWS_PER_PAGE=1000, **kwargs):
+                 SERVER_FROM_CURDIR='../', ROWS_PER_PAGE=1000, 
+                 returnstring = False, **kwargs):
     """
     Creates an html representation of tabular data, either from a tabarray or 
     an externa file (`including ``.hsv``, ``.csv``, ``.tsv``).  If no data is 
@@ -115,8 +117,9 @@ def tabular2html(fname, X=None, fin=None, title=None, printheader=False,
     """
 
     # Must write to an HTML file.
-    assert fname.endswith( '.html' ), 'fname must end in ".html".'
 
+    assert returnstring or fname.endswith( '.html' ), 'fname must end in ".html".'
+      
     if X is None:
         if fin is not None:
             if fin.lstrip('/').endswith('.hsv'):
@@ -179,7 +182,7 @@ def tabular2html(fname, X=None, fin=None, title=None, printheader=False,
             assert usecss == False
             CSSLINK = ''
     else:
-        if writecss == False:
+        if writecss == False or returnstring:
             CSSLINK = ''
         else:
             if not isinstance(writecss,str):
@@ -190,6 +193,9 @@ def tabular2html(fname, X=None, fin=None, title=None, printheader=False,
             CSSLINK = ('<link rel="stylesheet" type="text/css" href="' + '/'  + 
                        cssfile[len(SERVER_FROM_CURDIR):] + '"</link>')
 
+    if returnstring:
+        split = False
+    
     if not split or ROWS_PER_PAGE == 'all':
         ROWS_PER_PAGE = Num_Records + 1
 
@@ -210,91 +216,106 @@ def tabular2html(fname, X=None, fin=None, title=None, printheader=False,
         fromRow = section * ROWS_PER_PAGE  # Start record # for this section.
         toRow = min( fromRow + ROWS_PER_PAGE, Num_Records)  # End record #.
 
-        prefix = '/' + DirName(fname[len(SERVER_FROM_CURDIR):]) + '/'
+        if printheader and not returnstring:
+            prefix = '/' + DirName(fname[len(SERVER_FROM_CURDIR):]) + '/'
+        else:
+            prefix = ''
 
         # Open the output file for the section to fileobject 'f'.
-        with open( sectionfname, 'w' ) as f: 
-       
-            # Write out file header.
+        if not returnstring:
+            f = open(sectionfname,'w')
+        else:
+            f = tempfile.TemporaryFile('w+b')
+                     
+        # Write out file header.
+        if not returnstring:
             f.write('<html><META HTTP-EQUIV="Content-Type" '
-                    'CONTENT="text/html; charset=utf-8" /><head><title>' + 
-                    title + '</title>' + CSSLINK + '</head><body>\n' )
-            if printheader:
-                f.write('<p>Tabular File (page ' + str(section + 1) + ' of ' + 
-                        str(numSections) + ', rows ' + str(fromRow + 1) + 
-                        ' - ' + str(toRow) + '): ' + title + '</p>\n')
-                f.write('<p>page ')
-                if section > 0:
-                    f.write(' <a href="' + prefix + 
-                           basename(section2file(section - 1)) + '">prev</a> ')
-                if section < numSections - 1:
-                    f.write(' <a href="' + prefix + 
-                           basename(section2file(section + 1)) + '">next</a> ')
-                for page in range(numSections):
-                    f.write((' <a href="' + prefix + 
-                          basename(section2file(page)) + '">' + str(page + 1) + 
-                          '</a>') if page != section else ' ' + str(page + 1))
-                f.write( '</p>' )
+                'CONTENT="text/html; charset=utf-8" /><head><title>' + 
+                title + '</title>' + CSSLINK + '</head><body>\n' )
+        if printheader:
+            f.write('<p>Tabular File (page ' + str(section + 1) + ' of ' + 
+                    str(numSections) + ', rows ' + str(fromRow + 1) + 
+                    ' - ' + str(toRow) + '): ' + title + '</p>\n')
+            f.write('<p>page ')
+            if section > 0:
+                f.write(' <a href="' + prefix + 
+                       basename(section2file(section - 1)) + '">prev</a> ')
+            if section < numSections - 1:
+                f.write(' <a href="' + prefix + 
+                       basename(section2file(section + 1)) + '">next</a> ')
+            for page in range(numSections):
+                f.write((' <a href="' + prefix + 
+                      basename(section2file(page)) + '">' + str(page + 1) + 
+                      '</a>') if page != section else ' ' + str(page + 1))
+            f.write( '</p>' )
 
-            # Write out table with number of cols.
-            f.write('<table border="1" cellspacing="0" cellpadding="4">\n')
-            f.write('<col span="' + str(Num_Cols) + '" align="center">\n')
+        # Write out table with number of cols.
+        f.write('<table border="1" cellspacing="0" cellpadding="4">\n')
+        f.write('<col span="' + str(Num_Cols) + '" align="center">\n')
 
-            # Write out table header line.
-            f.write('<thead>')
-            if len(HdrNts) > 0:
-                for h in HdrNts:
-                    f.write(h + '\n')
-            f.write('<tr align="center">')
-            for name in names:
-                f.write('<th class="' + ColorStyles[0][name] + '">' + 
-                        cgi.escape(name) + '</th>')
-            f.write('</tr>')
-            f.write('</thead>\n')
+        # Write out table header line.
+        f.write('<thead>')
+        if len(HdrNts) > 0:
+            for h in HdrNts:
+                f.write(h + '\n')
+        f.write('<tr align="center">')
+        for name in names:
+            f.write('<th class="' + ColorStyles[0][name] + '">' + 
+                    cgi.escape(name) + '</th>')
+        f.write('</tr>')
+        f.write('</thead>\n')
 
-            # Write out each record in the section.
-            f.write( '<tbody>\n' )
-            if (len(names) > 1) or (fin != None and fin.endswith('.csv')):
-                for row in range( fromRow, toRow ):
-                    colorst = (' style="background-color:' + RowColors[row] + 
-                               '" ' if  RowColors[row] != '' else '')
-                    f.write('<tr align="center">')
-                    for (i, val) in enumerate(X[ row ]):
-                        #f.write('<td>' + cgi.escape(str(val)) + '</td>')
-                        f.write('<td ' + colorst + ' class="' + 
-                                ColorStyles[0][names[i]] + '">' + str(val) + 
-                                '</td>')
-                    f.write('</tr>\n')
-            else:
-                for row in range(fromRow, toRow):
-                    f.write('<tr align="center">')
-                    #f.write('<td>' + cgi.escape(str(X[row])) + '</td>')
-                    f.write('<td>' + str(X[row]) + '</td>')
-                    f.write('</tr>\n')
-            f.write('</tbody>\n')
+        # Write out each record in the section.
+        f.write( '<tbody>\n' )
+        if (len(names) > 1) or (fin != None and fin.endswith('.csv')):
+            for row in range( fromRow, toRow ):
+                colorst = (' style="background-color:' + RowColors[row] + 
+                           '" ' if  RowColors[row] != '' else '')
+                f.write('<tr align="center">')
+                for (i, val) in enumerate(X[ row ]):
+                    #f.write('<td>' + cgi.escape(str(val)) + '</td>')
+                    f.write('<td ' + colorst + ' class="' + 
+                            ColorStyles[0][names[i]] + '">' + str(val).replace('\n','<br/>') + 
+                            '</td>')
+                f.write('</tr>\n')
+        else:
+            for row in range(fromRow, toRow):
+                f.write('<tr align="center">')
+                #f.write('<td>' + cgi.escape(str(X[row])) + '</td>')
+                f.write('<td>' + str(X[row]).replace('\n','<br/>') + '</td>')
+                f.write('</tr>\n')
+        f.write('</tbody>\n')
 
-            f.write( '</table>' )
+        f.write( '</table>' )
 
-            # Write out hyperlinks to other sections.
-            if printheader:
-                f.write('<p>page ')
-                if section > 0:
-                    f.write(' <a href="' + prefix + 
-                           basename(section2file(section - 1)) + '">prev</a> ')
-                if section < numSections - 1:
-                    f.write(' <a href="' + prefix + 
-                           basename(section2file(section + 1)) + '">next</a> ')
-                for page in range(numSections):
-                    f.write((' <a href="' + prefix + 
-                             basename(section2file(page)) + '">' + 
-                             str(page + 1) + '</a>') if page != section 
-                                                     else ' ' + str(page + 1))
+        # Write out hyperlinks to other sections.
+        if printheader:
+            f.write('<p>page ')
+            if section > 0:
+                f.write(' <a href="' + prefix + 
+                       basename(section2file(section - 1)) + '">prev</a> ')
+            if section < numSections - 1:
+                f.write(' <a href="' + prefix + 
+                       basename(section2file(section + 1)) + '">next</a> ')
+            for page in range(numSections):
+                f.write((' <a href="' + prefix + 
+                         basename(section2file(page)) + '">' + 
+                         str(page + 1) + '</a>') if page != section 
+                                                 else ' ' + str(page + 1))
 
-                f.write('</p>')
+            f.write('</p>')
 
-            # End file.
+        # End file.
+        if not returnstring:
             f.write('</body></html>\n')
-
+    
+        if returnstring:
+            f.seek(0)
+            s = f.read()
+            f.close()
+            return s
+        else:
+            f.close()
 
 def FixCSSName(k):
     k.replace('.','__')
@@ -362,7 +383,9 @@ def EqualLevels(L, sdict):
     for i in range(len(L)):
         l1 = L[i]
         if l1 not in done:
-            LevelVals = sdict[l1][:]        # need to copy dictionary contents, otherwise LevelVals points to the dict!
+            # need to copy dictionary contents, 
+            # otherwise LevelVals points to the dict!
+            LevelVals = sdict[l1][:]        
             LevelKeys = [l1]
             done += [l1]
             for j in range(i, len(L)):
@@ -384,10 +407,8 @@ def MakeLine(names, L, sdict):
 
     for i in range(len(Diffs) - 1):
         if LN[Diffs[i] + 1] != '':
-            #s += '<td colspan = "' + str(Diffs[i + 1] - Diffs[i]) + '">' + (LN[Diffs[i] + 1] + '&nbsp &nbsp ')*(Diffs[i + 1] - Diffs[i] / 2) +  '</td>'
             s += ('<td colspan = "' + str(Diffs[i + 1] - Diffs[i]) + '">' + 
                   LN[Diffs[i] + 1]  +  '</td>')
-            #s +=  '<th>' + LN[Diffs[i] + 1] + '</th><th colspan="' + str(Diffs[i + 1] - Diffs[i] - 1) + '"><hr/></th>'
         else:
             s += '<th colspan = "' + str(Diffs[i + 1] - Diffs[i]) + '"></th>'
 
